/*
 ****************************************************************************************
 * Program        : PULSEPWM.CPP                                                        *
 * Description    : Demo program for pulse output function with PWM                     *
 * Boards Supp.   : PCI-1760                                                            *
 * APIs used      : DRV_DeviceOpen,DRV_DeviceClose, DRV_GetErrorMessage,                *
 *                  DRV_CounterReset, DRV_CounterPWMSetting, DRV_CounterPWMEnable       *
 * Revision       : 1.00                                                                *
 * Date           : 7/28/1999                   Advantech Co., Ltd.                      *
 ****************************************************************************************
 */
#include <windows.h>
#include <windef.h>
#include <stdio.h>
#include <conio.h>

#include "..\..\..\..\include\driver.h"

/******************************
 * Local function declaration *
 ******************************/
void ErrorHandler(DWORD dwErrCde);
void ErrorStop(long*, DWORD);

void main()
{
    DWORD  dwErrCde;
    ULONG  lDevNum;
    long   lDriverHandle;
    USHORT usChan, usOutCount, usChanMap;

    PT_CounterPWMSetting tCounterPWMSetting;

    //Step 1: Display hardware and software settings for running this example
    printf("Before running this example, please\n");
    printf("use the device installation utility to add the device.\n");

    //Step 2: Input parameters
    printf("\nPlease input parameters:");
    printf("\nDevice Number (check the device installation utility): ");
    scanf("%d", &lDevNum);
    printf("PWM Channel: ");
    scanf("%d", &usChan);
    printf("Pulse counts (0 for unlimit): ");
    scanf("%d", &usOutCount);

    //Step 3: Open device
    dwErrCde = DRV_DeviceOpen(lDevNum, &lDriverHandle);
    if (dwErrCde != SUCCESS)
    {
        ErrorHandler(dwErrCde);
        printf("Program terminated!\n");

        printf("Press any key to exit....");
        getch();
        exit(1);
    }

    // Step 4: Reset counter by DRV_CounterReset
    dwErrCde = DRV_CounterReset(lDriverHandle, usChan);
    if (dwErrCde != SUCCESS)
    {
        ErrorHandler(dwErrCde);
        printf("Program terminated!\n");

        dwErrCde = DRV_DeviceClose(&lDriverHandle);

        printf("Press any key to exit....");
        getch();
        exit(1);
    }

    // Step 5: Start counter operation by DRV_CounterPulseStart
    tCounterPWMSetting.Port = usChan;            // output channel
    tCounterPWMSetting.Period   = 10000;         // period: 1 second
    tCounterPWMSetting.HiPeriod =  5000;         // up cycle: 0.5 second
    tCounterPWMSetting.OutCount = usOutCount;    // output count
    tCounterPWMSetting.GateMode = 0;             // gate mode

    dwErrCde = DRV_CounterPWMSetting(lDriverHandle, &tCounterPWMSetting);
    if (dwErrCde != SUCCESS)
    {
        ErrorStop(&lDriverHandle, dwErrCde);
        return;
    }

    // Step 6: Enable PWM operation by DRV_CounterPWMEnable
    usChanMap = 1 << usChan;     //Transfer channel to bit map
    dwErrCde = DRV_CounterPWMEnable(lDriverHandle, usChanMap);
    if(dwErrCde != SUCCESS)
    {
        DRV_CounterReset(lDriverHandle, usChanMap);
        ErrorStop(&lDriverHandle, dwErrCde);
        return;
    }

    // Step 7: exit when pressing any key
    printf("\nPulse out on PWM%d with", usChan);
    printf("\nPeriod:      %5.3f second.", tCounterPWMSetting.Period / 10000);
    printf("\nUp cycle:    %5.4f second.", tCounterPWMSetting.HiPeriod/ 10000);
    printf("\nPulse count: %d\n",  tCounterPWMSetting.OutCount);
    printf("\nPress any key to stop running....");
    getch();

    // Step 8: Stop PWM output by DRV_CounterReset
    dwErrCde = DRV_CounterReset(lDriverHandle, usChanMap);

    // Step 9: Close device
    dwErrCde = DRV_DeviceClose(&lDriverHandle);
}//main

/**********************************************************************
 * Function: ErrorHandler
 *           Show the error message for the corresponding error code
 * input:    dwErrCde, IN, Error code
 * return:   none
 **********************************************************************/
void ErrorHandler(DWORD dwErrCde)
{
    char szErrMsg[180];

    DRV_GetErrorMessage(dwErrCde, szErrMsg);
    printf("\nError(%d): %s\n", dwErrCde & 0xffff, szErrMsg);
}//ErrorHandler

/**********************************************************************
 * Function:   ErrorStop
 *             Release all resource and terminate program if error occurs
 * Paramaters: pDrvHandle, IN/OUT, pointer to Driver handle
 *             dwErrCde, IN, Error code.
 * return:     none
 **********************************************************************/
void ErrorStop(long *pDrvHandle, DWORD dwErrCde)
{
    //Error message
    ErrorHandler(dwErrCde);
    printf("Program terminated!\n");

    //Close device
    DRV_DeviceClose(pDrvHandle);

    printf("Press any key to exit....");
    getch();
    exit(1);
}//ErrorStop
